﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Widget DraggingWidget { get; set; }

        public MainWindow()
        {
            InitializeComponent();

            Data data = new Data();
           
            NumericXAxis xAxis = new NumericXAxis();
            NumericYAxis yAxis = new NumericYAxis();
            xChart.Axes.Add(xAxis);
            xChart.Axes.Add(yAxis);
            ScatterSeries series = new ScatterSeries()
            {
                ItemsSource = data,
                XMemberPath = "ValueX",
                YMemberPath = "ValueY",
                XAxis = xAxis,
                YAxis = yAxis
            };
            xChart.Series.Add(series);

            xChart.SeriesMouseLeftButtonDown += XChart_SeriesMouseLeftButtonDown;           
            xChart.PreviewMouseMove += XChart_PreviewMouseMove;
            xChart.PreviewMouseUp += XChart_PreviewMouseUp;
        }

        private void XChart_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            this.DraggingWidget = null;
        }

        private void XChart_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            if (this.DraggingWidget != null)
            {
                Point p = e.GetPosition(xChart);

                this.DraggingWidget.ValueX = this.xChart.Axes.OfType<NumericXAxis>().First().GetUnscaledValue(p.X, this.xChart.WindowRect, this.xChart.ViewportRect);
                this.DraggingWidget.ValueY = this.xChart.Axes.OfType<NumericYAxis>().First().GetUnscaledValue(p.Y, this.xChart.WindowRect, this.xChart.ViewportRect);
            }
        }

        private void XChart_SeriesMouseLeftButtonDown(object sender, DataChartMouseButtonEventArgs e)
        {
            this.DraggingWidget = e.Item as Widget;
        }
    }

    public class Data : ObservableCollection<Widget>
    {
        public Data()
        {
            this.Add(new Widget() { ValueX = 1, ValueY = 1 });
            this.Add(new Widget() { ValueX = 2, ValueY = 2 });
            this.Add(new Widget() { ValueX = 3, ValueY = 3 });
        }
    }
    public class Widget : INotifyPropertyChanged
    {
        private double _ValueX;
        public double ValueX
        {
            get { return this._ValueX; }
            set
            {
                bool changed = this.ValueX != value;
                if (changed)
                {
                    this._ValueX = value;
                    this.RaisePropertyChanged();
                }
            }
        }
        private double _ValueY;
        public double ValueY
        {
            get { return this._ValueY; }
            set
            {
                bool changed = this.ValueY != value;
                if (changed)
                {
                    this._ValueY = value;
                    this.RaisePropertyChanged();
                }
            }
        }
        private void RaisePropertyChanged([CallerMemberName] string propertyName = "NONE")
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;
    }
}
